#include <windows.h>
#include "cdefs.h"
#include "htable.h"

extern cint boolean_True, object_Undefined;

void set_label(cint menu_handle, cint menu_item_ID, cint label)
{
  HMENU hmenu = object_as_int(menu_handle);
  UINT menuID = object_as_int(menu_item_ID);
  cstring menu_string = object_as_string(label);
  
  ModifyMenu(hmenu, menuID, GetMenuState(hmenu, menuID, MF_BYCOMMAND), menuID, menu_string);

  mem_free(menu_string);
  object_Dec_ref(menu_handle);
  object_Dec_ref(menu_item_ID);
  object_Dec_ref(label);
}

void enable_menu_item(cint menu_handle, cint menu_item_ID)
{
  HMENU hmenu = object_as_int(menu_handle);
  UINT menuID = object_as_int(menu_item_ID);

  EnableMenuItem(hmenu, menuID, MF_ENABLED);

  object_Dec_ref(menu_handle);
  object_Dec_ref(menu_item_ID);
}

void enable_menu(cint parent, cint menu_handle)
{
  HMENU hmenuparent = object_as_int(parent),
        hmenu = object_as_int(menu_handle);

  EnableMenuItem(hmenuparent, (UINT) hmenu, MF_ENABLED);

  object_Dec_ref(parent);
  object_Dec_ref(menu_handle);
}


void disable_menu(cint parent, cint menu_handle)
{
  HMENU hmenuparent = object_as_int(parent),
        hmenu = object_as_int(menu_handle);

  EnableMenuItem(hmenuparent, (UINT) hmenu, MF_GRAYED);

  object_Dec_ref(parent);
  object_Dec_ref(menu_handle);
}

void disable_menu_item(cint menu_handle, cint menu_item_ID)
{
  HMENU hmenu = object_as_int(menu_handle);
  UINT menuID = object_as_int(menu_item_ID);

  EnableMenuItem(hmenu, menuID, MF_GRAYED);

  object_Dec_ref(menu_handle);
  object_Dec_ref(menu_item_ID);
}

void check_set_state(cint menu_handle, cint menu_item_ID, cint check)
{
  HMENU hmenu = object_as_int(menu_handle);
  UINT menuID = object_as_int(menu_item_ID);

  CheckMenuItem(hmenu, menuID, ((check == boolean_True) ? MF_CHECKED : MF_UNCHECKED));

  object_Dec_ref(menu_handle);
  object_Dec_ref(menu_item_ID);
}

void add_separator(cint menu_handle)
{
  HMENU hmenu = object_as_int(menu_handle);
 
  AppendMenu(hmenu, MF_SEPARATOR, 0, NULL);

  object_Dec_ref(menu_handle);
}

void add_menu_item(cint menu_handle, cint menu_item_ID, cint label)
{
  HMENU hmenu = object_as_int(menu_handle);
  UINT menuID = object_as_int(menu_item_ID);
  cstring menu_string = object_as_string(label);

//add_to_hash_table(menuID, menu_item_ID);
  AppendMenu(hmenu, MF_STRING, menuID, menu_string);

  mem_free(menu_string);
  object_Dec_ref(menu_handle);
  object_Dec_ref(menu_item_ID);
  object_Dec_ref(label);
}

void del_menu_item(cint menu_handle, cint position)
{
  HMENU hmenu = object_as_int(menu_handle);
  UINT which_one = object_as_int(position);

//  DeleteMenu(hmenu, which_one, MF_BYPOSITION);
  RemoveMenu(hmenu, which_one, MF_BYPOSITION);
//remove_from_hash_table(GetMenuItemID(hmenu, which_one));

  object_Dec_ref(menu_handle);
  object_Dec_ref(position);
}

void add_menu(cint win_handle, cint menu_bar_handle, cint menu_handle, cint label) //, cint position)
{
  HWND hwnd = object_as_int(win_handle);  
  HMENU hmenubar = object_as_int(menu_bar_handle),
        hmenu = object_as_int(menu_handle);
//  cint pos = object_as_int(position);
  cstring menu_string = object_as_string(label);

  AppendMenu(hmenubar, MF_ENABLED | MF_POPUP, (UINT) hmenu, menu_string);
//  InsertMenu(hmenubar, pos, MF_BYPOSITION | MF_ENABLED | MF_POPUP, (UINT) hmenu, menu_string);
  DrawMenuBar(hwnd);

  mem_free(menu_string);
  object_Dec_ref(win_handle);
  object_Dec_ref(menu_bar_handle);
  object_Dec_ref(menu_handle);
  object_Dec_ref(label);
//  object_Dec_ref(position);
}

void del_menu(cint win_handle, cint menu_bar_handle, cint menu_handle)
{
  HWND hwnd = object_as_int(win_handle);  
  HMENU hmenubar = object_as_int(menu_bar_handle),
        hmenu = object_as_int(menu_handle);

//  DeleteMenu(hmenubar, hmenu, MF_BYCOMMAND);
  RemoveMenu(hmenubar, hmenu, MF_BYCOMMAND);
  DrawMenuBar(hwnd);

  object_Dec_ref(win_handle);
  object_Dec_ref(menu_bar_handle);
  object_Dec_ref(menu_handle);
}

cint create_menu_bar(cint target)
{
  HMENU menu_bar = CreateMenu();

//add_to_hash_table(menu_bar, object_as_int(target));

  object_Dec_ref(target);
 
  return object_integer_from(menu_bar);
}

cint create_menu(cint target)
{
  HMENU menu = CreatePopupMenu();

//add_to_hash_table(menu, object_as_int(target));

  object_Dec_ref(target);
 
  return object_integer_from(menu);
}

void set_menu_bar(cint window_handle, cint menu_handle)
{
  HMENU hmenu = object_as_int(menu_handle);
  HWND hwnd = object_as_int(window_handle);
  
  SetMenu(hwnd, (menu_handle == object_Undefined ? NULL : hmenu));

  object_Dec_ref(window_handle);
  object_Dec_ref(menu_handle);
}

void show_popup_menu(cptr menu_handle, cptr x_val, cptr y_val, cptr handle)
{
  HMENU hmenu = object_as_int(menu_handle);
  cint x = object_as_int(x_val),
       y = object_as_int(y_val);
  HWND hwnd = object_as_int(handle);
  RECT r;
  
  object_Dec_ref(menu_handle);
  object_Dec_ref(x_val);
  object_Dec_ref(y_val);
  object_Dec_ref(handle);

  GetWindowRect(hwnd, &r);
  while (GetParent(hwnd) != NULL)
    hwnd = GetParent(hwnd);
  TrackPopupMenu(hmenu, TPM_LEFTALIGN, x, y, 0, hwnd, &r);
}
