#include "htable.h"

//prime number based table sizes

const int table_sizes[] = {0, 1009, 2003, 3001, 4001, 5003, 6007,
			   7001, 8009, 9001, 10007, 11003, 12007,
			   13001, 14009, 15013, 16001, 17011,
			   18013, 19001, 20011, 21001, 22003,
			   23003, 24001, 25013, 26003, 27011,
			   28001, 29009, 30011, 31013, 32003};

const int num_indexes = sizeof(table_sizes) / sizeof(int);

void init_hash_table(hash_table *h) 
{
  int i;
  information_to_store *temp;
  
  if (h->index < num_indexes)
    h->current_table_size = table_sizes[h->index];
  else
    h->current_table_size += 1000;
  h->limit = (h->current_table_size * 3) / 4;
  h->num_entries = 0;
  temp = h->data;
  h->data = mem_malloc(h->current_table_size*sizeof(information_to_store));
  for (i = 0; i < h->current_table_size; i++)
  {
    h->data[i].key = NULL;
    h->data[i].object = NULL;
  }
  if (temp)
  {
    for (i = ((h->index < num_indexes) ? table_sizes[h->index-1] : h->current_table_size-1000); i > 0; i--)
      if (temp[i].object != NULL) 
        add_to_hash_table(h, temp[i].key, temp[i].object);
    mem_free(temp);
  }
}

//Inserts hwnd and object in table

void add_to_hash_table(hash_table *h, cint key, cptr object)
{
  int hash_key;
	
  if (h->num_entries >= h->limit)
  {
    h->index++;
    init_hash_table(h);
  }
  hash_key = key % h->current_table_size;
//  while (h->data[hash_key].key)
  while (h->data[hash_key].key && h->data[hash_key].object != NULL)
    hash_key = (hash_key + 1) % h->current_table_size;
  h->data[hash_key].key = key;
  h->data[hash_key].object = object;
  h->num_entries++;
}

//retrieves object from hash table
//assumes object is in table

cptr lookup_value_using_hash_table(hash_table *h, cint key)
{
  int hash_key;

  if (h->current_table_size)
    hash_key = key % h->current_table_size;
  else
    return NULL;
  while (h->data[hash_key].key != NULL)
    if (h->data[hash_key].key == key)  
      return h->data[hash_key].object;
    else
      hash_key = (hash_key + 1) % h->current_table_size;
  return NULL;
}

// `removes' entry from hash table

void remove_from_hash_table(hash_table *h, cint key)
{
  int hash_key;

  if (!h->current_table_size)
    return;

  hash_key = key % h->current_table_size;

  while (!(h->data[hash_key].key == key || h->data[hash_key].key == NULL))
    hash_key = (hash_key + 1) % h->current_table_size;
  h->data[hash_key].object = NULL; //if not necessary;
}
